<?php

namespace Drupal\campaign_kit\Campaign;

use Drupal\campaign_kit_core\Entity\Campaign;
use Drupal\campaign_kit_core\Entity\CampaignDonation;
use Drupal\campaign_kit_core\Entity\CampaignTransaction;
use Drupal\campaign_kit_core\Entity\Donation;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Locale\CountryManager;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\payment\Entity\Payment;
use Drupal\payment\Plugin\Payment\LineItem\PaymentLineItemManager;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodManager;
use Drupal\payment\Plugin\Payment\Type\PaymentTypeManager;

/**
 * Provide extra functions for campaigns.
 *
 * @package Drupal\campaign_kit\Campaign
 */
class CampaignService {
  use StringTranslationTrait;

  /**
   * @var array
   */
  protected $countries;

  /**
   * @var array
   */
  protected $stateUSOptions;

  /**
   * @var array
   */
  protected $stateCAOptions;

  /**
   * CurrentRouteMatch service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The service logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected $loggerChannelFactory;

  /**
   * The Payment Type Manager.
   *
   * @var \Drupal\payment\Entity\PaymentTypeManager
   */
  protected $paymentTypeManager;

  /**
   * The Payment Method manager.
   *
   * @var \Drupal\payment\Entity\PaymentMethodManager
   */
  protected $paymentMethodManager;

  /**
   * The payment line item manager.
   *
   * @var \Drupal\payment\Entity\PaymentLineItemManager
   */
  protected $paymentLineItemManager;

  /**
   * CampaignService constructor.
   *
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentRouteMatch
   *   The Current Route Match service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactory $loggerChannelFactory
   *   The service logger factory.
   * @param \Drupal\payment\Entity\PaymentTypeManager $paymentTypeManager
   *   The payment type manager.
   * @param \Drupal\payment\Entity\PaymentMethodManager $paymentMethodManager
   *   The Payment Method manager.
   * @param \Drupal\payment\Entity\PaymentLineItemManager $paymentLineItemManager
   *   The line item manager.
   */
  public function __construct(CurrentRouteMatch $currentRouteMatch,
    EntityTypeManagerInterface $entityTypeManager,AccountProxyInterface $currentUser,
    ConfigFactory $configFactory, LoggerChannelFactory $loggerChannelFactory,
    PaymentTypeManager $paymentTypeManager, PaymentMethodManager $paymentMethodManager,
    PaymentLineItemManager $paymentLineItemManager) {
      $this->currentRouteMatch = $currentRouteMatch;
      $this->entityTypeManager = $entityTypeManager;
      $this->currentUser = $currentUser;
      $this->configFactory = $configFactory;
      $this->loggerChannelFactory = $loggerChannelFactory;
      $this->paymentTypeManager = $paymentTypeManager;
      $this->paymentMethodManager = $paymentMethodManager;
      $this->paymentLineItemManager = $paymentLineItemManager;
      $this->countries = CountryManager::getStandardList();
      $this->stateUSOptions = $this->getUSStates();
      $this->stateCAOptions = $this->getCanadaStates();
  }

  /**
   * Create Campaign function.
   *
   * @param array $values
   *   Array values to create a Campaign entity.
   *
   * @return string
   *   Returns a object.
   */
  public function createCampaign(array $values = []) {
    return 'createCampaign';
  }

  /**
   * Create a Campaign Donation.
   *
   * @param array $values
   *   Values to create a Campaign Donation entity.
   *
   * @return \Drupal\campaign_kit_core\Entity\CampaignDonation
   *   Return a Campaign Donation entity.
   */
  public function createCampaignDonation(array $values = []) {
    // Create Campaign Donation entity.
    /** @var \Drupal\campaign_kit_core\Entity\CampaignDonation $campaignDonation */
    $campaignDonation = CampaignDonation::create($values);

    try {
      $campaignDonation->save();
    }
    catch (\Exception $e) {
      $message = $this->t("Your transaction did not save. Please contact the administrator.");
      $this->loggerChannelFactory->get('campaign_kit')->error($message);
      $message = $e->getMessage();
      $this->loggerChannelFactory->get('campaign_kit')->error($message);
    }

    return $campaignDonation;
  }

  /**
   * Create a Campaign Transaction.
   *
   * @param array $values
   *   Values to create a Campaign entity.
   *
   * @return \Drupal\campaign_kit_core\Entity\CampaignTransaction
   *   Return a Campaign Transaction entity.
   */
  public function createCampaignTransaction(array $values = []) {
    // Create Campaign Transaction entity.
    /** @var \Drupal\campaign_kit_core\Entity\CampaignTransaction $campaignTransaction */
    $campaignTransaction = CampaignTransaction::create($values);

    try {
      $campaignTransaction->save();
    }
    catch (\Exception $e) {
      $message = $this->t("Your transaction did not save. Please contact the administrator.");
      $this->loggerChannelFactory->get('campaign_kit')->error($message);
      $message = $e->getMessage();
      $this->loggerChannelFactory->get('campaign_kit')->error($message);
    }

    return $campaignTransaction;
  }

  /**
   * Create a Donation.
   *
   * @param array $values
   *   Values to create the Donation.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   Return a Donation entity.
   */
  public function createDonation(array $values = []) {
    // Create Donation entity.
    /** @var \Drupal\campaign_kit_core\Entity\Donation $campaignTransaction */
    $donation = Donation::create($values);

    try {
      $donation->save();
    }
    catch (\Exception $e) {
      $message = $this->t("Your donation did not save. Please contact the administrator.");
      $this->loggerChannelFactory->get('campaign_kit')->error($message);
      $message = $e->getMessage();
      $this->loggerChannelFactory->get('campaign_kit')->error($message);
    }

    return $donation;
  }

  /**
   * Add conditions to a view.
   *
   * @param $query
   * @param $fieldName
   * @param $lowestAmount
   * @param $highestAmount
   */
  public function alterExposedForm($query, $fieldName, $lowestAmount, $highestAmount) {
    if (isset($lowestAmount) && $lowestAmount != 0) {
      $query->where[0]['conditions'][4]['field'] = $fieldName . '.amount';
      $query->where[0]['conditions'][4]['value'] = (int) $lowestAmount;
      $query->where[0]['conditions'][4]['operator'] = '>=';
    }

    if (isset($highestAmount) && $highestAmount != 0) {
      $query->where[0]['conditions'][5]['field'] = $fieldName . '.amount';
      $query->where[0]['conditions'][5]['value'] = (int) $highestAmount;
      $query->where[0]['conditions'][5]['operator'] = '<=';
    }
  }

  /**
   * Return United States' states in an array.
   *
   * @return array
   */
  public function getUSStates() {
    return $stateOptions = [
      'AL' => $this->t('Alabama'),
      'AK' => $this->t('Alaska'),
      'AZ' => $this->t('Arizona'),
      'AR' => $this->t('Arkansas'),
      'CA' => $this->t('California'),
      'CO' => $this->t('Colorado'),
      'CT' => $this->t('Connecticut'),
      'DE' => $this->t('Delaware'),
      'DC' => $this->t('District of Columbia'),
      'FL' => $this->t('Florida'),
      'GA' => $this->t('Georgia'),
      'HI' => $this->t('Hawaii'),
      'ID' => $this->t('Idaho'),
      'IL' => $this->t('Illinois'),
      'IN' => $this->t('Indiana'),
      'IA' => $this->t('Iowa'),
      'KS' => $this->t('Kansas'),
      'KY' => $this->t('Kentucky'),
      'LA' => $this->t('Louisiana'),
      'ME' => $this->t('Maine'),
      'MD' => $this->t('Maryland'),
      'MA' => $this->t('Massachusetts'),
      'MI' => $this->t('Michigan'),
      'MN' => $this->t('Minnesota'),
      'MS' => $this->t('Mississippi'),
      'MO' => $this->t('Missouri'),
      'MY' => $this->t('Montana'),
      'NE' => $this->t('Nebraska'),
      'NV' => $this->t('Nevada'),
      'NH' => $this->t('New Hampshire'),
      'NJ' => $this->t('New Jersey'),
      'NM' => $this->t('New Mexico'),
      'NY' => $this->t('New York'),
      'NC' => $this->t('North Carolina'),
      'ND' => $this->t('North Dakota'),
      'OH' => $this->t('Ohio'),
      'OK' => $this->t('Oklahoma'),
      'OR' => $this->t('Oregon'),
      'PA' => $this->t('Pennsylvania'),
      'RI' => $this->t('Rhode Island'),
      'SC' => $this->t('South Carolina'),
      'SD' => $this->t('South Dakota'),
      'TN' => $this->t('Tennessee'),
      'TX' => $this->t('Texas'),
      'UT' => $this->t('Utah'),
      'VT' => $this->t('Vermont'),
      'VA' => $this->t('Virginia'),
      'WA' => $this->t('Washington'),
      'WV' => $this->t('West Virginia'),
      'WI' => $this->t('Wisconsin'),
      'WY' => $this->t('Wyoming'),
    ];
  }

  /**
   * Set United States' states in an array.
   *
   * @param $stateOptions
   */
  public function setUSStates($stateOptions) {
    $this->stateUSOptions = $stateOptions;
  }

  /**
   * Return Canada's states in an array.
   *
   * @return array
   */
  public function getCanadaStates() {
    return $stateOptions = [
      'AB' => $this->t('Alberta'),
      'BC' => $this->t('British Columbia'),
      'MB' => $this->t('Manitoba'),
      'NB' => $this->t('New Brunswick'),
      'NL' => $this->t('Newfoundland and Labrador'),
      'NS' => $this->t('Nova Scotia'),
      'ON' => $this->t('Ontario'),
      'PE' => $this->t('Prince Edward Island'),
      'QC' => $this->t('Quebec'),
      'SK' => $this->t('Saskatchewan'),
      'NT' => $this->t('Northwest Territories'),
      'NU' => $this->t('Nunavut'),
      'YT' => $this->t('Yukon'),
    ];
  }

  /**
   * Set Canada's states in an array.
   *
   * @param $stateCAOptions
   */
  public function setCanadaStates($stateCAOptions) {
    $this->stateCAOptions = $stateCAOptions;
  }

  /**
   * Get Campaign Object entity.
   *
   * @return \Drupal\campaign_kit_core\Entity\Campaign
   *   Return a Campaign entity.
   */
  public function getCampaignObject() {
    return $this->currentRouteMatch->getParameters()->get('campaign');
  }

  /**
   * Get the Entity Storage.
   *
   * @param $entityTypeId
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntityStorage($entityTypeId) {
    return $this->entityTypeManager->getStorage($entityTypeId);
  }

  /**
   * Get Countries.
   *
   * @return array
   */
  public function getCountries() {
    return $this->countries;
  }

  /**
   * Set Countries.
   *
   * @param $countries
   */
  public function setCountries($countries) {
    $this->countries = $countries;
  }

  /**
   * Get User ID.
   *
   * @return int
   */
  public function getUserId() {
    return $this->currentUser->id();
  }

  /**
   * Get Config Factory.
   *
   * @param $nameConfig
   *
   * @return \Drupal\Core\Config\Config|ImmutableConfig
   */
  public function getConfigFactory($nameConfig) {
    return $this->configFactory->get($nameConfig);
  }

  /**
   * Helper function to check Payment API Keys.
   *
   * @param string $config
   *   The config factory service.
   *
   * @return bool
   *   Boolean status, TRUE or FALSE.
   */
  public function checkTestPaymentApiKey($config) {
    $status = FALSE;
    $provider = '';

    // Get the provider.
    foreach ($this->paymentTypeManager->getDefinitions() as $pluginId => $definition) {
      $provider = $definition['provider'];
    }

    // We can add more payments here.
    switch ($provider) {
      case 'payment':
        echo "Do something here";
        break;

      case 'payment_stripe':
        // Do not forget to change this to live when it is in production.
        if ($config->get('environment') == 'test' && $config->get('apikey.test.secret')) {
          $status = TRUE;
        }
        break;
    }

    return $status;
  }

  /**
   * Helper function to create a Charge.
   *
   * @param string $config
   *   The config factory service.
   * @param string $token
   *   Stripe API token.
   *   Token is created using checkout or form element.
   * @param int $amount
   *   Amount for charge.
   * @param string $currency
   *   Code for currency.
   * @param string $campaignId
   *   Campaign ID.
   * @param string $campaignTitle
   *   Campaign title.
   * @param string $email
   *   Email.
   *
   * @return \Stripe\Charge
   *   Charge object.
   *
   * @throws \Stripe\Exception\ApiErrorException
   */
  public function createCharge($token, $amount, $currency, $campaignId, $campaignTitle, $email,
                               $askDonateSite, $newsletter) {

    $config = $this->getConfigFactory('campaign_kit.settings');
    $clientPlugin = $config->get('campaign_kit');
    $paymentMethodSelected = $clientPlugin['payment_metod_selected'];

    if ($paymentMethodSelected != '') {
      $split_method = explode(':', $paymentMethodSelected);
      $bundle = $split_method[0];

      /** @var \Drupal\payment\Entity\Payment */
      $paymentType = $this->paymentTypeManager->getDefinition($bundle);
      $payment = Payment::create(['bundle' => $bundle, 'payment_type' => $paymentType]);
      $paymentMethod = $this->paymentMethodManager->createInstance($paymentMethodSelected);
      $payment->setPaymentMethod($paymentMethod);

      // Metadata.
      $metadata = [
        "order_id" => $campaignId,
        "campaign_id" => $campaignId,
        "campaign_label" => $campaignTitle,
      ];
      if ($askDonateSite) {
        $metadata["donation_amount"] = $amount;
        $metadata["site_donation_amount"] = 5;
      }
      else {
        $metadata["donation_amount"] = $amount;
        $metadata["site_donation_amount"] = 0;
      }
      // Also make sure you have imported/enabled the currency via
      // the UI or it will be saved as null.
      $newsletter = ($newsletter) ? 'true' : 'false';
      $payment->setCurrencyCode($currency);
      $payment->set('field_newsletter', $newsletter);

      // Create a Line Item instance.
      $lineItems = [];
      $lineItems[] = $this->paymentLineItemManager->createInstance('payment_stripe_extra')
        ->setDescription($campaignTitle)
        ->setQuantity(1)
        ->setAmount($amount)
        ->setName($campaignTitle)
        ->setCurrencyCode($currency)
        ->setToken($token)
        ->setMetaData($metadata)
        ->setEmail($email);
      $payment->setLineItems($lineItems);

      if ($askDonateSite) {
        $lineItems[] = $this->paymentLineItemManager->createInstance('payment_basic')
          ->setDescription('Donation for site')
          ->setQuantity(1)
          ->setAmount(5)
          ->setName('donation_for_Site')
          ->setCurrencyCode($currency);
        $payment->setLineItems($lineItems);
      }

      try {
        $payment->execute();
        $paymentStatus = $payment->getPaymentStatus()->getPluginId();
        $paymentStatus = 'payment_success';
        if ($paymentStatus == 'payment_success') {
          $result = 'succeeded';
        }
        elseif ($paymentStatus == 'payment_failed') {
          $result = 'failed';
        }
      }
      catch (\Exception $e) {
        $message = "There was a problem processing the payment.";
        $this->loggerChannelFactory->get('campaign_service')->notice($message);
      }
      return $result;
    }
    else {
      $message = "A payment method must be selected in the campaign kit settings.";
      \Drupal::messenger()->addError(t($message));
      $this->loggerChannelFactory->get('campaign_service')->notice($message);
    }
  }

  /**
   * Set One-time and Monthly total amounts for a Campaign entity.
   *
   * @param $campaignId
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setOneTimeMonthlyTotalAmounts($campaignId) {
    $campaign = Campaign::load($campaignId);
    if ($campaign instanceof Campaign) {
      $campaign->set('total_one_time_amount', (int) $campaign->getRaisedAmountOneTime());
      $campaign->set('total_monthly_amount', (int) $campaign->getRaisedAmountMonthly());
      $campaign->save();
    }
  }

  /**
   * Get Payment Type.
   *
   * @return int|string
   */
  public function getPaymentType() {
    $type = '';
    unset($this->paymentTypeManager->getDefinitions()['payment_unavailable']);
    foreach ($this->paymentTypeManager->getDefinitions() as $pluginId => $definition) {
      $type = $pluginId;
    }

    return $type;
  }

  /**
   * Get Payment Method.
   *
   * @return int|string
   */
  public function getPaymentMethod() {
    $method = '';
    foreach ($this->paymentMethodManager->getDefinitions() as $pluginId => $definition) {
      if ($definition['active']) {
        $method = $pluginId;
      }
    }

    return $method;
  }

  /**
   * Get Payment Methods.
   */
  public function getPaymentMethods() {
    $methods = [];
    foreach ($this->paymentMethodManager->getDefinitions() as $pluginId => $definition) {
      if ($definition['active']) {
        $methods[$pluginId] = $definition['label'];
      }
    }
    return $methods;
  }

  /**
   * Create Payment Instance.
   *
   * @return \Drupal\payment\Entity\PaymentTypeInterface
   */
  public function createPaymentInstance() {
    return $this->paymentTypeManager->createInstance($this->getPaymentType());
  }
}
