<?php

namespace Drupal\campaign_kit\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\campaign_kit\Campaign\CampaignService;

/**
 * The Configuration Form.
 */
class ConfigurationForm extends FormBase {

  /**
   * The Messenger Service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Campaign Service.
   *
   * @var \Drupal\campaign_kit\Campaign\CampaignService
   */
  protected $campaignService;

  /**
   * ConfigurationForm constructor.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The Messenger Service.
   */
  public function __construct(MessengerInterface $messenger, CampaignService $campaignService) {
    $this->messenger = $messenger;
    $this->campaignService = $campaignService;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $messenger = $container->get('messenger');
    $campaignService = $container->get('campaign_kit.campaign_service');

    return new static(
      $messenger,
      $campaignService,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'payment_processor_form';
  }

  /**
   * Campaign Kit processor form.
   *
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // @todo Load the configuration of the payment plugin using the machine name.
    $campaignKitConfig = $this->configFactory()->get('campaign_kit.settings');
    $configurationForm = $campaignKitConfig->getRawData();

    // Campaign Kit Notification email.
    $form['campaign_kit_notification_email'] = [
      '#type' => 'email',
      '#title' => $this->t('Notification email'),
      '#default_value' => $configurationForm['campaign_kit']['notification_email'],
    ];

    // Currency.
    $form['currency'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Currency'),
      '#maxlength' => 6,
      '#size' => 6,
      '#default_value' => $configurationForm['campaign_kit']['currency'],
    ];

    // Campaigns Frequency allowed.
    $form['frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Campaigns frequency allowed'),
      '#default_value' => ($configurationForm['campaign_kit']['frequency'] != 'none' ? $configurationForm['campaign_kit']['frequency'] : 'empty_option'),
      '#options' => [
        'onetime' => $this->t('One-time'),
        'monthly' => $this->t('Monthly'),
        'both' => $this->t('Both'),
      ],
      '#empty_option' => $this->t('- None -'),
    ];

    $form['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Debug mode'),
      '#default_value' => $configurationForm['campaign_kit']['debug_mode'],
    ];

    // Non Campaign Donation Frequency allowed.
    $form['donation_frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Non-Campaign donation frequency allowed'),
      '#default_value' => ($configurationForm['campaign_kit']['donation_frequency'] != 'none' ? $configurationForm['campaign_kit']['donation_frequency'] : 'empty_option'),
      '#options' => [
        'onetime' => $this->t('One-time'),
        'monthly' => $this->t('Monthly'),
        'both' => $this->t('Both'),
      ],
    ];

    $form['image_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Image Location'),
      '#description' => $this->t('Set location where Image component stores files under the public folder. For example, "my_custom_folder" may store images in /sites/default/files/my_custom_folder.
'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $configurationForm['campaign_kit']['image_location'],
    ];

    $form['image_extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t(' Image component allowed file extensions'),
      '#description' => $this->t('Add extensions separated by spaces.'),
      '#required' => TRUE,
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $configurationForm['campaign_kit']['image_extensions'],
    ];
    $form['ask_for_site_donations'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ask for site donations.'),
      '#default_value' => $configurationForm['campaign_kit']['ask_for_site_donations'],
      '#attributes' => [
        'name' => 'ask_for_site_donations',
      ],
    ];
    $form['ask_for_site_donations_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Ask for site donations text'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $configurationForm['campaign_kit']['ask_for_site_donations_text'],
      '#states' => [
        'visible' => [
          ':input[name="ask_for_site_donations"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['ask_to_add_newsletter'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ask to add to newsletter.'),
      '#default_value' => $configurationForm['campaign_kit']['ask_newsletter'],
      '#attributes' => [
        'name' => 'ask_to_add_newsletter',
      ],
    ];
    $form['ask_to_add_newsletter_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Ask to add to newsletter text (for Donation Page only).'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $configurationForm['campaign_kit']['ask_newsletter_text'],
      '#states' => [
        'visible' => [
          ':input[name="ask_to_add_newsletter"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['default_newsletter_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Newsletter ID(s) (for Donation Page only).'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $configurationForm['campaign_kit']['default_newsletter_id'],
    ];
    $form['payment_method'] = [
      '#type' => 'select',
      '#title' => $this->t('Select Payment Method'),
      '#options' => $this->campaignService->getPaymentMethods(),
      '#empty_option' => $this->t('Please select'),
      '#required' => TRUE,
      '#default_value' => $configurationForm['campaign_kit']['payment_metod_selected'],
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * Saves configuration.
   *
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // @todo Save the configuration using the machine name of the payment plugin.
    $config = $this->configFactory()->getEditable('campaign_kit.settings');
    $config->set('campaign_kit.notification_email', $form_state->getValue('campaign_kit_notification_email'));
    $config->set('campaign_kit.currency', $form_state->getValue('currency'));
    $config->set('campaign_kit.frequency', $form_state->getValue('frequency'));
    $config->set('campaign_kit.debug_mode', $form_state->getValue('debug_mode'));
    // Single Donation Frequency Allowed.
    $config->set('campaign_kit.donation_frequency', $form_state->getValue('donation_frequency'));
    $config->set('campaign_kit.image_location', $form_state->getValue('image_location'));
    $config->set('campaign_kit.image_extensions', $form_state->getValue('image_extensions'));
    $config->set('campaign_kit.ask_for_site_donations', $form_state->getValue('ask_for_site_donations'));
    $config->set('campaign_kit.ask_for_site_donations_text', $form_state->getValue('ask_for_site_donations_text'));
    $config->set('campaign_kit.ask_newsletter', $form_state->getValue('ask_to_add_newsletter'));
    $config->set('campaign_kit.ask_newsletter_text', $form_state->getValue('ask_to_add_newsletter_text'));
    $config->set('campaign_kit.default_newsletter_id', $form_state->getValue('default_newsletter_id'));
    $config->set('campaign_kit.payment_metod_selected', $form_state->getValue('payment_method'));
    $config->save(TRUE);

    $this->messenger->addMessage($this->t('Configuration saved.'));
  }

}
